@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.5
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -27,6 +27,14 @@ import java.util.*;
  * in a specific {@linkplain Source source} document, which encompasses a {@linkplain #getStartTag() start tag},
  * an optional {@linkplain #getEndTag() end tag} and all {@linkplain #getContent() content} in between.
  * <p>
+ * Take the following HTML segment as an example:
+ * <p>
+ * <code>&lt;p&gt;This is a sample paragraph.&lt;/p&gt;</code>
+ * <p>
+ * The whole segment is represented by an <code>Element</code> object.  This is comprised of the {@link StartTag} "<code>&lt;p&gt;</code>",
+ * the {@link EndTag} "<code>&lt;/p&gt;</code>", as well as the text in between.
+ * An element may also contain other elements between its start and end tags.
+ * <p>
  * The term <i><a name="Normal">normal element</a></i> refers to an element having a {@linkplain #getStartTag() start tag}
  * with a {@linkplain StartTag#getStartTagType() type} of {@link StartTagType#NORMAL}.
  * This comprises all {@linkplain HTMLElements HTML elements} and <a href="HTMLElements.html#NonHTMLElement">non-HTML elements</a>.
@@ -47,6 +55,9 @@ import java.util.*;
  * <dl class="Separated">
  *  <dt><a name="SingleTag">Single Tag Element</a>:
  *   <dd>
+ *    Example:<br />
+ *    <code>&lt;img src="mypicture.jpg"&gt;</code>
+ *    <p>
  *    The element consists only of a single {@linkplain #getStartTag() start tag} and has no {@linkplain #getContent() element content}
  *    (although the start tag itself may have {@linkplain StartTag#getTagContent() tag content}).
  *    <br />{@link #getEndTag()}<code>==null</code>
@@ -70,6 +81,9 @@ import java.util.*;
  *    </ul>
  *  <dt><a name="ExplicitlyTerminated">Explicitly Terminated Element</a>:
  *   <dd>
+ *    Example:<br />
+ *    <code>&lt;p&gt;This is a sample paragraph.&lt;/p&gt;</code>
+ *    <p>
  *    The element consists of a {@linkplain #getStartTag() start tag}, {@linkplain #getContent() content},
  *    and an {@linkplain #getEndTag() end tag}.
  *    <br />{@link #getEndTag()}<code>!=null</code>.
@@ -86,6 +100,10 @@ import java.util.*;
  *    </ul>
  *  <dt><a name="ImplicitlyTerminated">Implicitly Terminated Element</a>:
  *   <dd>
+ *    Example:<br />
+ *    <code>&lt;p&gt;This text is included in the paragraph element even though no end tag is present.</code><br />
+ *    <code>&lt;p&gt;This is the next paragraph.</code>
+ *    <p>
  *    The element consists of a {@linkplain #getStartTag() start tag} and {@linkplain #getContent() content},
  *    but no {@linkplain #getEndTag() end tag}.
  *    <br />{@link #getEndTag()}<code>==null</code>.
@@ -130,7 +148,7 @@ import java.util.*;
  *         If the matching end tag is found, an <a href="#ExplicitlyTerminated">explicitly terminated element</a> is created.
  *        <li>
  *         If no matching end tag is found, the source document is not valid HTML and the incident is
- *         {@linkplain Source#setLogWriter(Writer) logged} as a missing required end tag.
+ *         {@linkplain Source#getLogger() logged} as a missing required end tag.
  *         In this situation a <a href="#SingleTag">single tag element</a> is created.
  *       </ul>
  *      <li>
@@ -192,7 +210,7 @@ import java.util.*;
  *         If the matching end tag is found, an <a href="#ExplicitlyTerminated">explicitly terminated element</a> is created.
  *        <li>
  *         If no matching end tag is found, the source document is not valid XML and the incident is
- *         {@linkplain Source#setLogWriter(Writer) logged} as a missing required end tag.
+ *         {@linkplain Source#getLogger() logged} as a missing required end tag.
  *         In this situation a <a href="#SingleTag">single tag element</a> is created.
  *       </ul>
  *     </ul>
@@ -210,7 +228,7 @@ import java.util.*;
  *      <li>
  *       If the matching end tag is found, an <a href="#ExplicitlyTerminated">explicitly terminated element</a> is created.
  *      <li>
- *       If no matching end tag is found, the missing required end tag is {@linkplain Source#setLogWriter(Writer) logged}
+ *       If no matching end tag is found, the missing required end tag is {@linkplain Source#getLogger() logged}
  *       and a <a href="#SingleTag">single tag element</a> is created.
  *     </ul>
  *   </ul>
@@ -298,10 +316,12 @@ public final class Element extends Segment implements HTMLElementName {
 						continue;
 					}
 					final Element childElement=childStartTag.getElement();
-					childElement.parentElement=this;
-					if (childElement.end>end && source.isLoggingEnabled()) source.log("Child element "+childElement.getDebugInfo()+" extends beyond end of parent "+getDebugInfo());
-					childElements.add(childElement);
+					if (childElement.end>end && source.logger.isInfoEnabled()) source.logger.info("Child "+childElement.getDebugInfo()+" extends beyond end of parent "+getDebugInfo());
 					childElement.getChildElements(childDepth);
+					if (childElement.parentElement==Element.NOT_CACHED) { // make sure element was not added as a child of a descendent element (can happen with overlapping elements)
+						childElement.parentElement=this;
+						childElements.add(childElement);
+					}
 					pos=childElement.end;
 				}
 			}
@@ -464,54 +484,16 @@ public final class Element extends Segment implements HTMLElementName {
 	public String getDebugInfo() {
 		if (this==NOT_CACHED) return "NOT_CACHED";
 		final StringBuffer sb=new StringBuffer();
-		sb.append("Element ").append(super.getDebugInfo()).append(": ");
-		startTag.appendStartTagDebugInfo(sb);
-		sb.append(endTag==null ? "(no end tag)" : "(with end tag)");
+		sb.append("Element ");
+		startTag.appendDebugTag(sb);
+		if (!isEmpty()) sb.append('-');
+		if (endTag!=null) sb.append(endTag);
+		sb.append(' ');
+		startTag.appendDebugTagType(sb);
+		sb.append(super.getDebugInfo());
 		return sb.toString();
 	}
 
-	/**
-	 * Returns the {@linkplain #getContent() content} text of the element.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as the {@link Segment} returned by the {@link #getContent()} method
-	 * now implements <code>CharSequence</code> and can be used directly in many cases.
-	 * Use {@link #getContent()}<code>.</code>{@link #toString() toString()} if a <code>String</code> is required.
-	 *
-	 * @return the content text of the element, or <code>null</code> if the element is {@linkplain #isEmpty() empty}.
-	 * @deprecated  Use {@link #isEmpty() isEmpty()}<code> ? null : </code>{@link #getContent()}<code>.</code>{@link #toString() toString()} instead.
-	 */
-	public String getContentText() {
-		return isEmpty() ? null : toString();
-	}
-
-	/**
-	 * Indicates whether the specified element name is an HTML {@linkplain HTMLElements#getBlockLevelElementNames() block-level element}.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as the {@link HTMLElements#getBlockLevelElementNames()} method now provides
- 	 * a complete set of the element names for which this method returns <code>true</code>.
-	 *
-	 * @param elementName  an element {@linkplain #getName() name}.
-	 * @return <code>true</code> if the specified element name is an HTML {@linkplain HTMLElements#getBlockLevelElementNames() block-level element}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link HTMLElements#getBlockLevelElementNames()}<code>.contains(elementName.toLowerCase())</code> instead.
-	 */
-	public static boolean isBlock(final String elementName) {
-		return HTMLElements.getBlockLevelElementNames().contains(elementName.toLowerCase());
-	}
-
-	/**
-	 * Indicates whether the specified element name is an HTML {@linkplain HTMLElements#getInlineLevelElementNames() inline-level element}.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as the {@link HTMLElements#getInlineLevelElementNames()} method now provides
- 	 * a complete set of the element names for which this method returns <code>true</code>.
-	 *
-	 * @param elementName  an element {@linkplain #getName() name}.
-	 * @return <code>true</code> if the specified element name is an HTML {@linkplain HTMLElements#getInlineLevelElementNames() inline-level element}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link HTMLElements#getInlineLevelElementNames()}<code>.contains(elementName.toLowerCase())</code> instead.
-	 */
-	public static boolean isInline(final String elementName) {
-		return HTMLElements.getInlineLevelElementNames().contains(elementName.toLowerCase());
-	}
-
 	int getContentEnd() {
 		return endTag!=null ? endTag.begin : end;
 	}
