@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.5
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -83,13 +83,14 @@ public abstract class FormControl extends Segment {
 			if (typeAttributeValue==null) return new InputFormControl(element,FormControlType.TEXT);
 			FormControlType formControlType=FormControlType.getFromInputElementType(typeAttributeValue.toLowerCase());
 			if (formControlType==null) {
-				if (element.source.isLoggingEnabled()) element.source.log(element.source.getRowColumnVector(element.begin).appendTo(new StringBuffer(200)).append(": INPUT control with unrecognised type \"").append(typeAttributeValue).append("\" assumed to be type \"text\"").toString());
+				if (element.source.logger.isInfoEnabled()) element.source.logger.info(element.source.getRowColumnVector(element.begin).appendTo(new StringBuffer(200)).append(": INPUT control with unrecognised type \"").append(typeAttributeValue).append("\" assumed to be type \"text\"").toString());
 				formControlType=FormControlType.TEXT;
 			}
 			if (formControlType==FormControlType.TEXT) return new InputFormControl(element,formControlType);
 			if (formControlType==FormControlType.CHECKBOX || formControlType==FormControlType.RADIO) return new RadioCheckboxFormControl(element,formControlType);
 			if (formControlType==FormControlType.SUBMIT) return new SubmitFormControl(element,formControlType);
 			if (formControlType==FormControlType.IMAGE) return new ImageSubmitFormControl(element);
+			if (formControlType==FormControlType.NON_FORM_CONTROL) return null;
 			// formControlType is HIDDEN || PASSWORD || FILE
 			return new InputFormControl(element,formControlType);
 		} else if (tagName==Tag.SELECT) {
@@ -97,7 +98,7 @@ public abstract class FormControl extends Segment {
 		} else if (tagName==Tag.TEXTAREA) {
 			return new TextAreaFormControl(element);
 		} else if (tagName==Tag.BUTTON) {
-			return element.getAttributes().getRawValue(Attribute.TYPE).equalsIgnoreCase("submit") ? new SubmitFormControl(element,FormControlType.BUTTON) : null;
+			return "submit".equalsIgnoreCase(element.getAttributes().getRawValue(Attribute.TYPE)) ? new SubmitFormControl(element,FormControlType.BUTTON) : null;
 		} else {
 			return null;
 		}
@@ -599,7 +600,7 @@ public abstract class FormControl extends Segment {
 			super(element,formControlType,true);
 			if (elementContainer.predefinedValue==null) {
 				elementContainer.predefinedValue=CHECKBOX_NULL_DEFAULT_VALUE;
-				if (element.source.isLoggingEnabled()) element.source.log(element.source.getRowColumnVector(element.begin).appendTo(new StringBuffer(200)).append(": compulsory \"value\" attribute of ").append(formControlType).append(" control \"").append(name).append("\" is missing, assuming the value \"").append(CHECKBOX_NULL_DEFAULT_VALUE).append('"').toString());
+				if (element.source.logger.isInfoEnabled()) element.source.logger.info(element.source.getRowColumnVector(element.begin).appendTo(new StringBuffer(200)).append(": compulsory \"value\" attribute of ").append(formControlType).append(" control \"").append(name).append("\" is missing, assuming the value \"").append(CHECKBOX_NULL_DEFAULT_VALUE).append('"').toString());
 			}
 		}
 		public boolean setValue(final CharSequence value) {
@@ -787,8 +788,10 @@ public abstract class FormControl extends Segment {
 	}
 
 	private static void findAll(final Segment segment, final ArrayList list, final String tagName) {
-		for (final Iterator i=segment.findAllElements(tagName).iterator(); i.hasNext();)
-			list.add(((Element)i.next()).getFormControl());
+		for (final Iterator i=segment.findAllElements(tagName).iterator(); i.hasNext();) {
+			final FormControl formControl=((Element)i.next()).getFormControl();
+			if (formControl!=null) list.add(formControl);
+		}
 	}
 
 	private static CharSequence getString(final char ch, final int length) {
@@ -808,7 +811,7 @@ public abstract class FormControl extends Segment {
 			missingOrBlank="blank";
 		}
 		final Source source=getElement().source;
-		if (source.isLoggingEnabled()) source.log(getElement().source.getRowColumnVector(getElement().begin).appendTo(new StringBuffer(200)).append(": compulsory \"name\" attribute of ").append(formControlType).append(" control is ").append(missingOrBlank).toString());
+		if (source.logger.isInfoEnabled()) source.logger.info(getElement().source.getRowColumnVector(getElement().begin).appendTo(new StringBuffer(200)).append(": compulsory \"name\" attribute of ").append(formControlType).append(" control is ").append(missingOrBlank).toString());
 	}
 
 	private static final void addValueTo(final Collection collection, final CharSequence value) {
