@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.5
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -139,7 +139,6 @@ public final class FormControlType {
 	private boolean hasPredefinedValue;
 	private boolean submit;
 
-	private static final HashMap ID_MAP=new HashMap(16,1.0F); // 12 types in total
 	private static final HashMap INPUT_ELEMENT_TYPE_MAP=new HashMap(11,1.0F); // 8 input element types in total
 
 	/**
@@ -156,7 +155,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = true</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType BUTTON=new FormControlType("button",Tag.BUTTON,true,true).register();
+	public static final FormControlType BUTTON=new FormControlType("button",HTMLElementName.BUTTON,true,true);
 
 	/**
 	 * The form control type given to a <a href="#checkbox-control">checkbox</a> control.
@@ -171,7 +170,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType CHECKBOX=new FormControlType("checkbox",Tag.INPUT,true,false).register();
+	public static final FormControlType CHECKBOX=new FormControlType("checkbox",HTMLElementName.INPUT,true,false);
 
 	/**
 	 * The form control type given to a <a href="#file-select-control">file select</a> control.
@@ -202,7 +201,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType FILE=new FormControlType("file",Tag.INPUT,false,false).register();
+	public static final FormControlType FILE=new FormControlType("file",HTMLElementName.INPUT,false,false);
 
 	/**
 	 * The form control type given to a <a href="#hidden-control">hidden</a> control.
@@ -219,7 +218,7 @@ public final class FormControlType {
 	 * Note that {@link #hasPredefinedValue()} returns <code>false</code> for this control type
 	 * because the value of hidden fields is usually set via server or client side scripting.
 	 */
-	public static final FormControlType HIDDEN=new FormControlType("hidden",Tag.INPUT,false,false).register();
+	public static final FormControlType HIDDEN=new FormControlType("hidden",HTMLElementName.INPUT,false,false);
 
 	/**
 	 * The form control type given to a <a href="#submit-button">submit button</a> control implemented using an
@@ -252,7 +251,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = true</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType IMAGE=new FormControlType("image",Tag.INPUT,true,true).register();
+	public static final FormControlType IMAGE=new FormControlType("image",HTMLElementName.INPUT,true,true);
 
 	/**
 	 * The form control type given to a <a href="#text-input-control">text input</a> control implemented using an
@@ -269,7 +268,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType PASSWORD=new FormControlType("password",Tag.INPUT,false,false).register();
+	public static final FormControlType PASSWORD=new FormControlType("password",HTMLElementName.INPUT,false,false);
 
 	/**
 	 * The form control type given to a <a href="#radio-button-control">radio button</a> control.
@@ -284,7 +283,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType RADIO=new FormControlType("radio",Tag.INPUT,true,false).register();
+	public static final FormControlType RADIO=new FormControlType("radio",HTMLElementName.INPUT,true,false);
 
 	/**
 	 * The form control type given to a <a href="#menu-control">menu</a> control implemented using a
@@ -321,7 +320,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType SELECT_MULTIPLE=new FormControlType("select_multiple",Tag.SELECT,true,false).register();
+	public static final FormControlType SELECT_MULTIPLE=new FormControlType("select_multiple",HTMLElementName.SELECT,true,false);
 
 	/**
 	 * The form control type given to a <a href="#menu-control">menu</a> control implemented using a
@@ -353,7 +352,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType SELECT_SINGLE=new FormControlType("select_single",Tag.SELECT,true,false).register();
+	public static final FormControlType SELECT_SINGLE=new FormControlType("select_single",HTMLElementName.SELECT,true,false);
 
 	/**
 	 * The form control type given to a <a href="#submit-button">submit button</a> control implemented using an
@@ -370,7 +369,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = true</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType SUBMIT=new FormControlType("submit",Tag.INPUT,true,true).register();
+	public static final FormControlType SUBMIT=new FormControlType("submit",HTMLElementName.INPUT,true,true);
 
 	/**
 	 * The form control type given to a <a href="#text-input-control">text input</a> control implemented using an
@@ -387,7 +386,7 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType TEXT=new FormControlType("text",Tag.INPUT,false,false).register();
+	public static final FormControlType TEXT=new FormControlType("text",HTMLElementName.INPUT,false,false);
 
 	/**
 	 * The form control type given to a <a href="#text-input-control">text input</a> control implemented using a
@@ -403,7 +402,22 @@ public final class FormControlType {
 	 *    <code>{@link #isSubmit()} = false</code><br />
 	 * </dl>
 	 */
-	public static final FormControlType TEXTAREA=new FormControlType("textarea",Tag.TEXTAREA,false,false).register();
+	public static final FormControlType TEXTAREA=new FormControlType("textarea",HTMLElementName.TEXTAREA,false,false);
+
+	static final FormControlType NON_FORM_CONTROL=new FormControlType("non_form_control",HTMLElementName.INPUT,false,false);
+
+	static {
+		INPUT_ELEMENT_TYPE_MAP.put("checkbox",CHECKBOX);
+		INPUT_ELEMENT_TYPE_MAP.put("file",FILE);
+		INPUT_ELEMENT_TYPE_MAP.put("hidden",HIDDEN);
+		INPUT_ELEMENT_TYPE_MAP.put("image",IMAGE);
+		INPUT_ELEMENT_TYPE_MAP.put("password",PASSWORD);
+		INPUT_ELEMENT_TYPE_MAP.put("radio",RADIO);
+		INPUT_ELEMENT_TYPE_MAP.put("submit",SUBMIT);
+		INPUT_ELEMENT_TYPE_MAP.put("text",TEXT);
+		INPUT_ELEMENT_TYPE_MAP.put("button",NON_FORM_CONTROL);
+		INPUT_ELEMENT_TYPE_MAP.put("reset",NON_FORM_CONTROL);
+	}
 
 	private FormControlType(final String formControlTypeId, final String elementName, final boolean hasPredefinedValue, final boolean submit) {
 		this.formControlTypeId=formControlTypeId;
@@ -412,12 +426,6 @@ public final class FormControlType {
 		this.submit=submit;
 	}
 
-	private FormControlType register() {
-		ID_MAP.put(formControlTypeId,this);
-		if (elementName==Tag.INPUT) INPUT_ELEMENT_TYPE_MAP.put(formControlTypeId,this);
-		return this;
-	}
-
 	/**
 	 * Returns the {@linkplain Element#getName() name} of the {@link Element} that constitues this form control type.
 	 * @return the {@linkplain Element#getName() name} of the {@link Element} that constitues this form control type.
@@ -462,125 +470,6 @@ public final class FormControlType {
 		return submit;
 	}
 
-	/**
-	 * Returns the {@linkplain Element#getName() name} of the {@link Element} that constitues this form control type.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 and replaced with the exactly equivalent
-	 * {@link #getElementName()} method for aesthetical reasons.
-	 *
-	 * @return the {@linkplain Element#getName() name} of the {@link Element} that constitues this form control type.
-	 * @deprecated  Use {@link #getElementName() getElementName()} instead.
-	 */
-	public String getTagName() {
-		return elementName;
-	}
-
-	/**
-	 * Indicates whether any <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#current-value">value</a>
-	 * submitted by this type of control is predefined in the HTML and typically not modified by the user or server/client scripts.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 and replaced with the exactly equivalent
-	 * {@link #hasPredefinedValue()} method for aesthetical reasons.
-	 *
-	 * @return <code>true</code> if any <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#current-value">value</a> submitted by this type of control is predefined in the HTML and typically not modified by the user or server/client scripts, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #hasPredefinedValue() hasPredefinedValue()} instead.
-	 */
-	public boolean isPredefinedValue() {
-		return hasPredefinedValue;
-	}
-
-	/**
-	 * Indicates whether more than one control of this type with the same <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> can be <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#successful-controls">successful</a>.
-	 * <p>
-	 * Returns <code>false</code> only for the {@link #RADIO}, {@link #SUBMIT}, {@link #BUTTON}, and {@link #IMAGE} instances.
-	 * <p>
-	 * Note that before version 1.4.1 this method also returned <code>false</code> for the {@link #SELECT_SINGLE} instance.
-	 * This was a bug resulting from confusion as to whether each <code>OPTION</code> element in a
-	 * <code>SELECT</code> element constituted a control (since it is possible for multiple options to be
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#successful-controls">successful</a>)
-	 * or only the <code>SELECT</code> element as a whole.
-	 * Now that the control is clearly defined as the entire <code>SELECT</code> element,
-	 * it is clear that multiple {@link #SELECT_SINGLE} controls with the same name result in multiple values.
-	 * <p>
-	 * Because this may not be immediately intuitive, and the method is no longer used internally,
-	 * this method has been deprecated as of version 2.0 to avoid any further confusion.
-	 *
-	 * @return <code>true</code> if more than one control of this type with the same <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> can be <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#successful-controls">successful</a>, otherwise <code>false</code>.
-	 * @deprecated  Use the more useful {@link FormField#allowsMultipleValues()} method instead.
-	 */
-	public boolean allowsMultipleValues() {
-		return !(this==RADIO || isSubmit());
-	}
-
-	/**
-	 * Returns a string which identifies this form control type.
-	 * <p>
-	 * This is the same as the control type's static field name in lower case, which is one of<br />
-	 * <code>button</code>, <code>checkbox</code>, <code>file</code>, <code>hidden</code>, <code>image</code>,
-	 * <code>password</code>, <code>radio</code>, <code>select_multiple</code>, <code>select_single</code>,
-	 * <code>submit</code>, <code>text</code>, or <code>textarea</code>.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as it has no practical use.
-	 *
-	 * @return a string which identifies this form control type.
-	 * @deprecated  Use {@link #toString() toString()} instead.
-	 */
-	public String getFormControlTypeId() {
-		return formControlTypeId;
-	}
-
-	/**
-	 * Returns the {@link FormControlType} with the specified {@linkplain #getFormControlTypeId() ID}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as it has no practical use.
-	 * <p>
-	 * @param formControlTypeId  the ID of a form control type.
-	 * @return the {@link FormControlType} with the specified ID, or <code>null</code> if no such control exists.
-	 * @see #getFormControlTypeId()
-	 * @deprecated  no replacement
-	 */
-	public static FormControlType get(final String formControlTypeId) {
-		return (FormControlType)ID_MAP.get(formControlTypeId);
-	}
-
-	/**
-	 * Returns an array containing the additional field names submitted if a control of this type with the specified <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> is <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#successful-controls">successful</a>.
-	 * <p>
-	 * Returns <code>null</code> for all control types except {@link #IMAGE}.
-	 * It relates to the extra <code><i>name</i>.x</code> and <code><i>name</i>.y</code> data submitted when a pointing device is used to activate an IMAGE control.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as it is no longer used internally and
-	 * has no practical use as a public method.
-	 *
-	 * @param name  the <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> of a form control.
-	 * @return an array containing the additional field names submitted if a control of this type with the specified <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#control-name">name</a> is <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#successful-controls">successful</a>, or <code>null</code> if none.
-	 * @deprecated  no replacement
-	 */
-	public String[] getAdditionalSubmitNames(final String name) {
-		if (this!=IMAGE) return null;
-		final String[] names=new String[2];
-		names[0]=name+".x";
-		names[1]=name+".y";
-		return names;
-	}
-
-	/**
-	 * Indicates whether an HTML tag with the specified name is potentially a form <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#form-controls">control</a>.
-	 * <p>
-	 * Returns <code>true</code> if the specified tag name is one of
-	 * "input", "textarea", "button" or "select" (ignoring case).
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as it is no longer used internally and
-	 * has no practical use as a public method.
-	 *
-	 * @param tagName  the name of an HTML tag.
-	 * @return <code>true</code> if an HTML tag with the specified name is potentially a form <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#form-controls">control</a>, otherwise <code>false</code>.
-	 * @deprecated  no replacement
-	 */
-	public static boolean isPotentialControl(final String tagName) {
-		return tagName.equalsIgnoreCase(Tag.INPUT) || tagName.equalsIgnoreCase(Tag.TEXTAREA) || tagName.equalsIgnoreCase(Tag.BUTTON) || tagName.equalsIgnoreCase(Tag.SELECT);
-	}
-
 	/**
 	 * Returns a string representation of this object useful for debugging purposes.
 	 * @return a string representation of this object useful for debugging purposes.
