@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.5
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -25,6 +25,8 @@ import java.util.*;
 /**
  * Represents a segment of a {@link Source} document.
  * <p>
+ * Many of the <a href="Tag.html#TagSearchMethods">tag search methods</a> are defined in this class.
+ * <p>
  * The <i>span</i> of a segment is defined by the combination of its begin and end character positions.
  */
 public class Segment implements Comparable, CharSequence {
@@ -92,6 +94,7 @@ public class Segment implements Comparable, CharSequence {
 	 * @return <code>true</code> if the specified object is equal to this <code>Segment</code>, otherwise <code>false</code>.
 	 */
 	public final boolean equals(final Object object) {
+		if (this==object) return true;
 		if (object==null || !(object instanceof Segment)) return false;
 		final Segment segment=(Segment)object;
 		return segment.begin==begin && segment.end==end && segment.source==source;
@@ -158,99 +161,38 @@ public class Segment implements Comparable, CharSequence {
 	}
 
 	/**
-	 * Extracts the text content of this segment.
+	 * Performs a simple rendering of the HTML markup in this segment into text.
 	 * <p>
-	 * This method removes all of the tags from the segment and
-	 * {@linkplain CharacterReference#decodeCollapseWhiteSpace(CharSequence) decodes the result, collapsing all white space}.
-	 * <p>
-	 * See the documentation of the {@link #extractText(boolean includeAttributes)} method for more details.
-	 * <p>
-	 * This is equivalent to calling {@link #extractText(boolean) extractText(false)}.
-	 *
-	 * @return the text content of this segment.
+	 * The output can be configured by setting any number of properties on the returned {@link Renderer} instance before
+	 * {@linkplain Renderer#writeTo(Writer) obtaining its output}.
+	 * 
+	 * @return an instance of {@link Renderer} based on this segment.
+	 * @see #getTextExtractor()
 	 */
-	public String extractText() {
-		return extractText(false);
+	public Renderer getRenderer() {
+		return new Renderer(this);
 	}
 
 	/**
-	 * Extracts the text content of this segment.
+	 * Extracts the textual content from the HTML markup of this segment.
 	 * <p>
-	 * This method removes all of the tags from the segment and
-	 * {@linkplain CharacterReference#decodeCollapseWhiteSpace(CharSequence) decodes the result, collapsing all white space}.
-	 * Tags are also converted to whitespace unless they belong to an
-	 * {@linkplain HTMLElements#getInlineLevelElementNames() inline-level} element.
-	 * An exception to this is the {@link HTMLElementName#BR BR} element, which is also converted to whitespace despite being an inline-level element.
+	 * The output can be configured by setting properties on the returned {@link TextExtractor} instance before
+	 * {@linkplain TextExtractor#writeTo(Writer) obtaining its output}.
 	 * <p>
-	 * Text inside {@link HTMLElementName#SCRIPT SCRIPT} and {@link HTMLElementName#STYLE STYLE} elements contained within this segment
-	 * is ignored.
-	 * <p>
-	 * Specifying a value of <code>true</code> as an argument to the <code>includeAttributes</code> parameter causes the values of 
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-title">title</a>,
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/objects.html#adef-alt">alt</a>,
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-label-OPTION">label</a>, and
-	 * <a target="_blank" href="http://www.w3.org/TR/html401/struct/tables.html#adef-summary">summary</a>
-	 * attributes of {@linkplain StartTagType#NORMAL normal} tags to be included in the extracted text.
-	 * <p>
-	 * <dl>
-	 *  <dt>Example:</dt>
-	 *  <dd>source segment "<code>&lt;div&gt;&lt;b&gt;O&lt;/b&gt;ne&lt;/div&gt;&lt;div&gt;&lt;b&gt;T&lt;/b&gt;&lt;script&gt;//a&nbsp;script&nbsp;&lt;/script&gt;wo&lt;/div&gt;</code>"
-	 *   produces the text "<code>One Two</code>".
-	 * </dl>
-	 * <p>
-	 * Note that in version 2.1, no tags were converted to whitespace and text inside {@link HTMLElementName#SCRIPT SCRIPT} and
-	 * {@link HTMLElementName#STYLE STYLE} elements was included.  The example above produced the text "<code>OneT//a script wo</code>".
-	 *
-	 * @param includeAttributes  indicates whether the values of <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-title">title</a>, <a target="_blank" href="http://www.w3.org/TR/html401/struct/objects.html#adef-alt">alt</a>, <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-label-OPTION">label</a>, and <a target="_blank" href="http://www.w3.org/TR/html401/struct/tables.html#adef-summary">summary</a> attributes are included.
-	 * @return the text content of this segment.
+	 * @return an instance of {@link TextExtractor} based on this segment.
+	 * @see #getRenderer()
 	 */
-	public String extractText(final boolean includeAttributes) {
-		final StringBuffer sb=new StringBuffer(length());
-		int textBegin=begin;
-		// use findAllTags().iterator() instead of source.findNextTag(textBegin) to take advantage of allTags cache in Source object
-		for (final Iterator i=findAllTags().iterator(); i.hasNext();) {
-			final Tag tag=(Tag)i.next();
-			final int textEnd=tag.begin;
-			if (textEnd<textBegin) continue;
-			while (textBegin<textEnd) sb.append(source.charAt(textBegin++));
-			if (tag.getTagType()==StartTagType.NORMAL) {
-				if (tag.name==HTMLElementName.SCRIPT || tag.name==HTMLElementName.STYLE) {
-					final EndTag endTag=source.findNextEndTag(tag.end,tag.name,EndTagType.NORMAL);
-					if (endTag!=null) {
-						textBegin=endTag.end;
-						while (i.hasNext() && i.next()!=endTag) {}
-						continue;
-					}
-				}
-				if (includeAttributes) {
-					final StartTag startTag=(StartTag)tag;
-					// add title attribute:
-					final Attribute titleAttribute=startTag.getAttributes().get("title");
-					if (titleAttribute!=null) sb.append(' ').append(titleAttribute.getValueSegment()).append(' ');
-					// add alt attribute (APPLET, AREA, IMG and INPUT elements):
-					final Attribute altAttribute=startTag.getAttributes().get("alt");
-					if (altAttribute!=null) sb.append(' ').append(altAttribute.getValueSegment()).append(' ');
-					// add label attribute (OPTION and OPTGROUP elements):
-					final Attribute labelAttribute=startTag.getAttributes().get("label");
-					if (labelAttribute!=null) sb.append(' ').append(labelAttribute.getValueSegment()).append(' ');
-					// add summary attribute (TABLE element):
-					final Attribute summaryAttribute=startTag.getAttributes().get("summary");
-					if (summaryAttribute!=null) sb.append(' ').append(summaryAttribute.getValueSegment()).append(' ');
-					// don't bother with the prompt attribute from the ININDEX element as the element is deprecated and very rarely used.
-				}
-			}
-			// Treat tags not belonging to inline-level elements as whitespace:
-			if (tag.getName()==HTMLElementName.BR || !HTMLElements.getInlineLevelElementNames().contains(tag.getName())) sb.append('\n');
-			textBegin=tag.end;
-		}
-		while (textBegin<end) sb.append(source.charAt(textBegin++));
-		final String decodedText=CharacterReference.decodeCollapseWhiteSpace(sb);
-		return decodedText;
+	public TextExtractor getTextExtractor() {
+		return new TextExtractor(this);
 	}
 
 	/**
 	 * Returns a list of all {@link Tag} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
+	 * The {@link Source#fullSequentialParse()} method should be called after construction of the {@link Source} object
+	 * if this method is to be used on a large proportion of the source.
+	 * It is called automatically if this method is called on the {@link Source} object itself.
+	 * <p>
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 *
 	 * @return a list of all {@link Tag} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
@@ -283,6 +225,10 @@ public class Segment implements Comparable, CharSequence {
 	/**
 	 * Returns a list of all {@link StartTag} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
+	 * The {@link Source#fullSequentialParse()} method should be called after construction of the {@link Source} object
+	 * if this method is to be used on a large proportion of the source.
+	 * It is called automatically if this method is called on the {@link Source} object itself.
+	 * <p>
 	 * See the {@link Tag} class documentation for more details about the behaviour of this method.
 	 *
 	 * @return a list of all {@link StartTag} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
@@ -343,8 +289,18 @@ public class Segment implements Comparable, CharSequence {
 	 * <p>
 	 * The returned list may include an element that extends beyond the end of this segment, as long as it begins within this segment.
 	 * <p>
+	 * An element found at the start of this segment is included in the list.
+	 * Note however that if this segment <i>is</i> an {@link Element}, the overriding {@link Element#getChildElements()} method is called instead,
+	 * which only returns the children of the element.
+	 * <p>
+	 * Calling <code>getChildElements()</code> on an <code>Element</code> is usually more efficient than calling it on a <code>Segment</code>.
+	 * <p>
 	 * The objects in the list are all of type {@link Element}.
 	 * <p>
+	 * The {@link Source#fullSequentialParse()} method should be called after construction of the {@link Source} object
+	 * if this method is to be used on a large proportion of the source.
+	 * It is called automatically if this method is called on the {@link Source} object itself.
+	 * <p>
 	 * See the {@link Source#getChildElements()} method for more details.
 	 *
 	 * @return the a list of the immediate children of this segment in the document element hierarchy, guaranteed not <code>null</code>.
@@ -377,6 +333,10 @@ public class Segment implements Comparable, CharSequence {
 	/**
 	 * Returns a list of all {@link Element} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * <p>
+	 * The {@link Source#fullSequentialParse()} method should be called after construction of the {@link Source} object
+	 * if this method is to be used on a large proportion of the source.
+	 * It is called automatically if this method is called on the {@link Source} object itself.
+	 * <p>
 	 * The elements returned correspond exactly with the start tags returned in the {@link #findAllStartTags()} method.
 	 *
 	 * @return a list of all {@link Element} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
@@ -432,6 +392,30 @@ public class Segment implements Comparable, CharSequence {
 		return elements;
 	}
 
+	/**
+	 * Returns a list of all {@link Element} objects with the specified attribute name/value pair 
+	 * that are {@linkplain #encloses(Segment) enclosed} by this segment.
+	 * <p>
+	 * The elements returned correspond exactly with the start tags returned in the {@link #findAllStartTags(String attributeName, String value, boolean valueCaseSensitive)} method.
+	 *
+	 * @param attributeName  the attribute name (case insensitive) to search for, must not be <code>null</code>.
+	 * @param value  the value of the specified attribute to search for, must not be <code>null</code>.
+	 * @param valueCaseSensitive  specifies whether the attribute value matching is case sensitive.
+	 * @return a list of all {@link Element} objects with the specified attribute name/value pair that are {@linkplain #encloses(Segment) enclosed} by this segment.
+	 */
+	public List findAllElements(final String attributeName, final String value, final boolean valueCaseSensitive) {
+		final List startTags=findAllStartTags(attributeName,value,valueCaseSensitive);
+		if (startTags.isEmpty()) return Collections.EMPTY_LIST;
+		final ArrayList elements=new ArrayList(startTags.size());
+		for (final Iterator i=startTags.iterator(); i.hasNext();) {
+			final StartTag startTag=(StartTag)i.next();
+			final Element element=startTag.getElement();
+			if (element.end>end) break;
+			elements.add(element);
+		}
+		return elements;
+	}
+
 	/**
 	 * Returns a list of all {@link CharacterReference} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
 	 * @return a list of all {@link CharacterReference} objects that are {@linkplain #encloses(Segment) enclosed} by this segment.
@@ -483,18 +467,46 @@ public class Segment implements Comparable, CharSequence {
 	/**
 	 * Causes the this segment to be ignored when parsing.
 	 * <p>
-	 * This method is usually used to exclude {@linkplain TagType#isServerTag() server tags} or other non-HTML segments from the source text
-	 * so that they do not interfere with the parsing of the surrounding HTML.
+	 * Ignored segments are treated as blank spaces by the parsing mechanism, but are included as normal text in all other functions.
 	 * <p>
-	 * This is necessary because many server tags are used as attribute values and in other places within
-	 * HTML tags, and very often contain characters that prevent the parser from recognising the surrounding tag.
+	 * This method was originally the only means of preventing {@linkplain TagType#isServerTag() server tags} located inside
+	 * {@linkplain StartTagType#NORMAL normal} tags from interfering with the parsing of the tags.
+	 * The most common scenario is where the {@linkplain Attributes attributes} of a normal tag uses server tags to dynamically set the values of the attributes.
+	 * <p>
+	 * As of version 2.4 it is no longer necessary to use this method to ignore {@linkplain StartTagType#SERVER_COMMON common server tags} inside normal tags,
+	 * as the attributes parser now automatically ignores common server tags.
+	 * <p>
+	 * As of version 2.5 it is also unnecessary to use this method to ignore the contents of {@link HTMLElementName#SCRIPT SCRIPT} elements,
+	 * as the parser automatically ignores this content when performing a {@linkplain Source#fullSequentialParse() full sequential parse}.
+	 * <p>
+	 * This leaves only a few scenarios where calling this method still provides a significant benefit.
+	 * <p>
+	 * One such case is where XML-style server tags are used inside {@linkplain StartTagType#NORMAL normal} tags.
+	 * Here is an example using an XML-style JSP tag:
+	 * <blockquote class="code"><code>&lt;a href="&lt;i18n:resource path="/Portal"/&gt;?BACK=TRUE"&gt;back&lt;/a&gt;</code></blockquote>
+	 * The first double-quote of <code>"/Portal"</code> will be interpreted as the end quote for the <code>href</code> attribute,
+	 * as there is no way for the parser to recognise the <code>il8n:resource</code> element as a server tag.
+	 * Such use of XML-style server tags inside {@linkplain StartTagType#NORMAL normal} tags is generally seen as bad practice,
+	 * but it is nevertheless valid JSP.  The only way to ensure that this library is able to parse the normal tag surrounding it is to
+	 * find these server tags first and call the <code>ignoreWhenParsing</code> method to ignore them before parsing the rest of the document.
+	 * <p>
+	 * It is important to understand the difference between ignoring the segment when parsing and removing the segment completely.
+	 * Any text inside a segment that is ignored when parsing is treated by most functions as content, and as such is included in the output of
+	 * tools such as {@link TextExtractor} and {@link Renderer}.
+	 * <p>
+	 * To remove segments completely, create an {@link OutputDocument} and call its {@link OutputDocument#remove(Segment) remove(Segment)} or
+	 * {@link OutputDocument#replaceWithSpaces(int,int) replaceWithSpaces(int begin, int end)} method for each segment.
+	 * Then create a new source document using {@link Source#Source(CharSequence) new Source(outputDocument.toString())}
+	 * and perform the desired operations on this new source object.
+	 * <p>
+	 * Calling this method after the {@link Source#fullSequentialParse()} method has been called is not permitted and throws an <code>IllegalStateException</code>.
 	 * <p>
 	 * Any tags appearing in this segment that are found before this method is called will remain in the {@linkplain Source#getCacheDebugInfo() tag cache},
 	 * and so will continue to be found by the <a href="Tag.html#TagSearchMethods">tag search methods</a>.
 	 * If this is undesirable, the {@link Source#clearCache()} method can be called to remove them from the cache.
 	 * Calling the {@link Source#fullSequentialParse()} method after this method clears the cache automatically.
 	 * <p>
-	 * For efficiency reasons, this method should be called on all segments that need to be ignored without calling
+	 * For best performance, this method should be called on all segments that need to be ignored without calling
 	 * any of the <a href="Tag.html#TagSearchMethods">tag search methods</a> in between.
 	 *
 	 * @see Source#ignoreWhenParsing(Collection segments)
@@ -619,81 +631,28 @@ public class Segment implements Comparable, CharSequence {
 	}
 
 	/**
-	 * Indicates whether this segment is a {@link Tag} of type {@link StartTagType#COMMENT}.
+	 * Extracts the textual content from the HTML markup of this segment.
 	 * <p>
-	 * This method has been deprecated as of version 2.0 as it is not a robust method of checking whether an HTML comment spans this segment.
+ 	 * This method has been deprecated as of version 2.4 and replaced with the {@link #getTextExtractor()} method.
 	 *
-	 * @return <code>true</code> if this segment is a {@link Tag} of type {@link StartTagType#COMMENT}, otherwise <code>false</code>.
-	 * @deprecated  Use <code>this instanceof </code>{@link Tag}<code> && ((Tag)this).</code>{@link Tag#getTagType() getTagType()}<code>==</code>{@link StartTagType#COMMENT} instead.
+	 * @return the textual content from the HTML markup of this segment.
+	 * @deprecated  Use {@link #getTextExtractor()}<code>.</code>{@link TextExtractor#toString() toString()} instead.
 	 */
-	public boolean isComment() {
-		return false; // overridden in StartTag
+	public String extractText() {
+		return new TextExtractor(this).toString();
 	}
 
 	/**
-	 * Returns a list of all {@link StartTag} objects representing HTML {@linkplain StartTagType#COMMENT comments} that are {@linkplain #encloses(Segment) enclosed} by this segment.
+	 * Extracts the textual content from the HTML markup of this segment.
 	 * <p>
-	 * This method has been deprecated as of version 2.0 in favour of the more generic {@link #findAllTags(TagType)} method.
+ 	 * This method has been deprecated as of version 2.4 and replaced with the {@link #getTextExtractor()} method.
 	 *
-	 * @return a list of all {@link StartTag} objects representing HTML {@linkplain StartTagType#COMMENT comments} that are {@linkplain #encloses(Segment) enclosed} by this segment.
-	 * @deprecated  Use {@link #findAllTags(TagType) findAllTags}<code>(</code>{@link StartTagType#COMMENT}<code>)</code> instead.
+	 * @param includeAttributes  specifies whether the values of <a target="_blank" href="http://www.w3.org/TR/html401/struct/global.html#adef-title">title</a>, <a target="_blank" href="http://www.w3.org/TR/html401/struct/objects.html#adef-alt">alt</a>, <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#adef-label-OPTION">label</a>, and <a target="_blank" href="http://www.w3.org/TR/html401/struct/tables.html#adef-summary">summary</a> attributes are included in the output.
+	 * @return the textual content from the HTML markup of this segment.
+	 * @deprecated  Use {@link #getTextExtractor()}<code>.</code>{@link TextExtractor#setIncludeAttributes(boolean) setIncludeAttributes(includeAttributes)}<code>.</code>{@link TextExtractor#toString() toString()} instead.
 	 */
-	public List findAllComments() {
-		return findAllTags(StartTagType.COMMENT);
-	}
-
-	/**
-	 * Returns the source text of this segment.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as it now duplicates the functionality of the {@link #toString()} method.
-	 *
-	 * @return the source text of this segment.
-	 * @deprecated  Use {@link #toString() toString()} instead.
-	 */
-	public String getSourceText() {
-		return toString();
-	}
-
-	/**
-	 * Returns the source text of this segment without {@linkplain #isWhiteSpace(char) white space}.
-	 * <p>
-	 * All leading and trailing white space is omitted, and any sections of internal white space are replaced by a single space.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as it is no longer used internally and
-	 * has no practical use as a public method.
-	 * It is similar to the new {@link CharacterReference#decodeCollapseWhiteSpace(CharSequence)} method, but
-	 * does not {@linkplain CharacterReference#decode(CharSequence) decode} the text after collapsing the white space.
-	 * <p>
-	 * @return the source text of this segment without white space.
-	 * @deprecated  Use the more useful {@link CharacterReference#decodeCollapseWhiteSpace(CharSequence)} method instead.
-	 */
-	public final String getSourceTextNoWhitespace() {
-		return appendCollapseWhiteSpace(new StringBuffer(length()),this).toString();
-	}
-
-	/**
-	 * Returns a list of <code>Segment</code> objects representing every word in this segment separated by {@linkplain #isWhiteSpace(char) white space}.
-	 * Note that any markup contained in this segment is regarded as normal text for the purposes of this method.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as it has no discernable use.
-	 *
-	 * @return a list of <code>Segment</code> objects representing every word in this segment separated by white space.
-	 * @deprecated  no replacement
-	 */
-	public final List findWords() {
-		final ArrayList words=new ArrayList();
-		int wordBegin=-1;
-		for (int i=begin; i<end; i++) {
-			if (isWhiteSpace(source.charAt(i))) {
-				if (wordBegin==-1) continue;
-				words.add(new Segment(source,wordBegin,i));
-				wordBegin=-1;
-			} else {
-				if (wordBegin==-1) wordBegin=i;
-			}
-		}
-		if (wordBegin!=-1) words.add(new Segment(source, wordBegin,end));
-		return words;
+	public String extractText(final boolean includeAttributes) {
+		return new TextExtractor(this).setIncludeAttributes(includeAttributes).toString();
 	}
 
 	/**
