@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.5
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -149,7 +149,7 @@ public final class StartTag extends Tag {
 			element=new Element(source,this,endTag);
 			if (endTag!=null) {
 				if (endTag.element!=Element.NOT_CACHED)
-					if (source.isLoggingEnabled()) source.log(source.getRowColumnVector(endTag.begin).appendTo(new StringBuffer(200).append("End tag ").append(endTag).append(" at ")).append(" terminates more than one element").toString()); // presumably impossible, but log it just in case
+					if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(endTag.begin).appendTo(new StringBuffer(200).append("End tag ").append(endTag).append(" at ")).append(" terminates more than one element").toString()); // presumably impossible, but log it just in case
 				endTag.element=element;
 			}
 		}
@@ -450,225 +450,27 @@ public final class StartTag extends Tag {
 
 	public String getDebugInfo() {
 		final StringBuffer sb=new StringBuffer();
-		appendStartTagDebugInfo(sb);
+		appendDebugTag(sb);
+		sb.append(' ');
+		appendDebugTagType(sb);
 		sb.append(super.getDebugInfo());
 		return sb.toString();
 	}
 
-	StringBuffer appendStartTagDebugInfo(final StringBuffer sb) {
-		sb.append('"').append(name).append("\" ");
-		if (startTagType!=StartTagType.NORMAL) sb.append('(').append(startTagType.getDescription()).append(") ");
+	StringBuffer appendDebugTag(final StringBuffer sb) {
+		if (startTagType==StartTagType.NORMAL && getAttributes().isEmpty()) {
+			sb.append(this);
+		} else {
+			sb.append('<').append(getNameSegment()).append(' ');
+			if (isEmptyElementTag()) sb.append('/');
+			sb.append(startTagType.getClosingDelimiter());
+		}
 		return sb;
 	}
 
-	/**
-	 * Returns an XML representation of this start tag.
-	 * <p>
-	 * This method has been deprecated as of version 2.2 and replaced with the exactly equivalent {@link #tidy()} method.
-	 *
-	 * @return an XML representation of this start tag, or the {@linkplain Segment#toString() source text} if it is of a {@linkplain #getStartTagType() type} that does not {@linkplain StartTagType#hasAttributes() have attributes}
-	 * @deprecated  Use {@link #tidy()} instead.
-	 */
-	public String regenerateHTML() {
-		return tidy();
-	}
-
-	/**
-	 * Indicates whether a matching end tag is <i>optional</i> according to the HTML 4.01 specification.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 and replaced with the {@link HTMLElements#getEndTagOptionalElementNames()}
-	 * static method.
-	 * <p>
-	 * This property is only relevant to start tags forming part of an <a href="Element.html#HTML">HTML element</a>
-	 * and returns <code>false</code> in all other cases.
-	 *
-	 * @return  <code>true</code> if a matching end tag is <i>optional</i> according to the HTML 4.01 specification, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getStartTagType()}<code>==</code>{@link StartTagType#NORMAL}<code> && </code>{@link HTMLElements#getEndTagOptionalElementNames()}<code>.contains(</code>{@link #getName() getName()}<code>)</code> instead.
-	 */
-	public boolean isEndTagOptional() {
-		return getStartTagType()==StartTagType.NORMAL && HTMLElements.getEndTagOptionalElementNames().contains(name);
-	}
-
-	/**
-	 * Returns the end tag that corresponds to this start tag.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as it has existed mainly for backward compatability with version 1.0.
-	 * <p>
-	 * The {@link #getElement()} method is much more useful as it determines the span of the
-	 * element even if the end tag is {@linkplain #isEndTagOptional() optional} and is not present in the source document.
-	 * <p>
-	 * This method on the other hand just returns <code>null</code> in the above case, revealing nothing about where the element ends.
-	 *
-	 * @return the end tag that corresponds to this start tag, or <code>null</code> if it does not exist in the source document.
-	 * @deprecated  Use {@link #getElement()}<code>.</code>{@link Element#getEndTag() getEndTag()} instead.
-	 */
-	public EndTag findEndTag() {
-		return getElement().getEndTag();
-	}
-
-	/**
-	 * Returns the {@link FormControlType} of this start tag.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as it is no longer used internally and
-	 * has no practical use as a public method.
-	 *
-	 * @return the form control type of this start tag, or <code>null</code> if it is not a <a target="_blank" href="http://www.w3.org/TR/html401/interact/forms.html#form-controls">control</a>.
-	 * @see Element#getFormControl()
-	 * @deprecated  Use {@link #getFormControl()}<code>.</code>{@link FormControl#getFormControlType() getFormControlType()} instead.
-	 */
-	public FormControlType getFormControlType() {
-		final FormControl formControl=getFormControl();
-		if (formControl==null) return null;
-		return formControl.getFormControlType();
-	}
-
-	/**
-	 * Returns the segment containing the text that immediately follows this start tag up until the start of the following tag.
-	 * <p>
-	 * Guaranteed not <code>null</code>.
-	 * <p>
- 	 * This method has been deprecated as of version 2.0 as it is no longer used internally and
-	 * has no practical use as a public method.
-	 *
-	 * @return the segment containing the text that immediately follows this start tag up until the start of the following tag.
-	 * @deprecated  Use {@link Segment#Segment(Source,int,int) new Segment}<code>(source,</code>{@link #getEnd() getEnd()}<code>,</code>{@link #findNextTag() findNextTag()}<code>.</code>{@link #getBegin() getBegin()}<code>)</code> instead.
-	 */
-	public Segment getFollowingTextSegment() {
-		int endData=source.getParseText().indexOf('<',end);
-		if (endData==-1) endData=source.end;
-		return new Segment(source,end,endData);
-	}
-
-	/**
-	 * Indicates whether the start tag is a server tag.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if the start tag is a server tag, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}.{@link TagType#isServerTag() isServerTag()} instead.
-	 */
-	public boolean isServerTag() {
-		return getTagType().isServerTag();
-	}
-
-	/**
-	 * Indicates whether this start tag is of type {@link StartTagType#COMMENT}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag is of type {@link StartTagType#COMMENT}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link StartTagType#COMMENT} instead.
-	 */
-	public boolean isComment() {
-		return startTagType==StartTagType.COMMENT;
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@link StartTagType#XML_PROCESSING_INSTRUCTION} or is any other tag starting with "&lt;?".
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@link StartTagType#XML_PROCESSING_INSTRUCTION} or is any other tag starting with "&lt;?", otherwise <code>false</code>.
-	 * @deprecated  Use <code>charAt(1)=='?'</code> instead for backward compatibility.
-	 */
-	public boolean isProcessingInstruction() {
-		return charAt(1)=='?';
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@link StartTagType#XML_DECLARATION}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@link StartTagType#XML_DECLARATION}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link StartTagType#XML_DECLARATION} instead.
-	 */
-	public boolean isXMLDeclaration() {
-		return startTagType==StartTagType.XML_DECLARATION;
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@link StartTagType#DOCTYPE_DECLARATION}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@link StartTagType#DOCTYPE_DECLARATION}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link StartTagType#DOCTYPE_DECLARATION} instead.
-	 */
-	public boolean isDocTypeDeclaration() {
-		return startTagType==StartTagType.DOCTYPE_DECLARATION;
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@linkplain StartTagType#SERVER_COMMON}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@linkplain StartTagType#SERVER_COMMON}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link StartTagType#SERVER_COMMON} instead.
-	 */
-	public boolean isCommonServerTag() {
-		return startTagType==StartTagType.SERVER_COMMON;
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@link PHPTagTypes#PHP_STANDARD}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@link PHPTagTypes#PHP_STANDARD}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link PHPTagTypes#PHP_STANDARD} instead.
-	 */
-	public boolean isPHPTag() {
-		return startTagType==PHPTagTypes.PHP_STANDARD;
-	}
-
-	/**
-	 * Indicates whether this start tag would be {@linkplain MasonTagTypes#isParsedByMason(TagType) parsed by a Mason server}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag would be {@linkplain MasonTagTypes#isParsedByMason(TagType) parsed by a Mason server}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link MasonTagTypes}<code>.</code>{@link MasonTagTypes#isParsedByMason(TagType) isParsedByMason}<code>(</code>{@link #getTagType() getTagType()}<code>)</code> instead.
-	 */
-	public boolean isMasonTag() {
-		return startTagType==StartTagType.SERVER_COMMON || startTagType==MasonTagTypes.MASON_NAMED_BLOCK || startTagType==MasonTagTypes.MASON_COMPONENT_CALL || startTagType==MasonTagTypes.MASON_COMPONENT_CALLED_WITH_CONTENT;
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@linkplain MasonTagTypes#MASON_NAMED_BLOCK}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@linkplain MasonTagTypes#MASON_NAMED_BLOCK}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link MasonTagTypes#MASON_NAMED_BLOCK} instead.
-	 */
-	public boolean isMasonNamedBlock() {
-		return startTagType==MasonTagTypes.MASON_NAMED_BLOCK;
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@linkplain MasonTagTypes#MASON_COMPONENT_CALL}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@linkplain MasonTagTypes#MASON_COMPONENT_CALL}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link MasonTagTypes#MASON_COMPONENT_CALL} instead.
-	 */
-	public boolean isMasonComponentCall() {
-		return startTagType==MasonTagTypes.MASON_COMPONENT_CALL;
-	}
-
-	/**
-	 * Indicates whether this start tag has a {@linkplain #getTagType() type} of {@linkplain MasonTagTypes#MASON_COMPONENT_CALLED_WITH_CONTENT}.
-	 * <p>
-	 * This method has been deprecated as of version 2.0 as its functionality is now easily performed without a dedicated method.
-	 *
-	 * @return <code>true</code> if this start tag has a {@linkplain #getTagType() type} of {@linkplain MasonTagTypes#MASON_COMPONENT_CALLED_WITH_CONTENT}, otherwise <code>false</code>.
-	 * @deprecated  Use {@link #getTagType() getTagType()}<code>==</code>{@link MasonTagTypes#MASON_COMPONENT_CALLED_WITH_CONTENT} instead.
-	 */
-	public boolean isMasonComponentCalledWithContent() {
-		return startTagType==MasonTagTypes.MASON_COMPONENT_CALLED_WITH_CONTENT;
+	StringBuffer appendDebugTagType(final StringBuffer sb) {
+		if (startTagType!=StartTagType.NORMAL) sb.append('(').append(startTagType.getDescription()).append(") ");
+		return sb;
 	}
 
 	private EndTag findEndTagInternal() {
@@ -694,7 +496,7 @@ public final class StartTag extends Tag {
 			if (HTMLElements.END_TAG_REQUIRED_NESTING_FORBIDDEN_SET.contains(name)) {
 				final StartTag nextStartTag=source.findNextStartTag(end,name);
 				if (nextStartTag==null || nextStartTag.begin>nextEndTag.begin) return nextEndTag;
-				if (source.isLoggingEnabled()) source.log(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append("StartTag at ")).append(" missing required end tag - invalid nested start tag encountered before end tag").toString());
+				if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append("StartTag at ")).append(" missing required end tag - invalid nested start tag encountered before end tag").toString());
 				// Terminate the element at the start of the invalidly nested start tag.
 				// This is how IE and Mozilla treat illegally nested A elements, but other elements may vary.
 				return new EndTag(source,nextStartTag.begin,nextStartTag.begin,EndTagType.NORMAL,name);
@@ -702,7 +504,7 @@ public final class StartTag extends Tag {
 			final Segment[] findResult=findEndTag(nextEndTag,checkForEmptyElementTag);
 			if (findResult!=null) return (EndTag)findResult[0];
 		}
-		if (source.isLoggingEnabled()) source.log(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append("StartTag at ")).append(" missing required end tag").toString());
+		if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(begin).appendTo(new StringBuffer(200).append("StartTag at ")).append(" missing required end tag").toString());
 		return null;
 	}
 
@@ -778,30 +580,38 @@ public final class StartTag extends Tag {
 		}
 	}
 	
-	static StartTag findNext(final Source source, int pos, final String attributeName, final String value, final boolean valueCaseSensitive) {
-		if (value==null) throw new IllegalArgumentException();
-		final char[] valueCharArray=value.toLowerCase().toCharArray();
+	static StartTag findNext(final Source source, final int pos, final String attributeName, final String value, final boolean valueCaseSensitive) {
+		if (value==null || attributeName.length()==0) throw new IllegalArgumentException();
+		// Determine whether to perform the text search on the name or value:
+		// - perform the text search on the value if it is >= 3 chars long.
+		// - have to perform the text search on the name if the value is zero length.
+		// - perform the text search on the name if the name >= 3 chars long, otherwise on the value.
+		final String searchString=value.length()>=3 || (value.length()>0 && attributeName.length()<3) ? value : attributeName;
+		final char[] searchCharArray=searchString.toLowerCase().toCharArray();
 		final ParseText parseText=source.getParseText();
-		while (pos<source.end) {
-			pos=parseText.indexOf(valueCharArray,pos);
-			if (pos==-1) return null;
-			final Tag tag=source.findEnclosingTag(pos);
+		int searchPos=pos;
+		while (searchPos<source.end) {
+			searchPos=parseText.indexOf(searchCharArray,searchPos);
+			if (searchPos==-1) return null;
+			final Tag tag=source.findEnclosingTag(searchPos);
 			if (tag==null || !(tag instanceof StartTag)) {
-				pos++;
+				searchPos++;
 				continue;
 			}
-			final StartTag startTag=(StartTag)tag;
-			if (startTag.getAttributes()!=null) {
-				final String attributeValue=startTag.getAttributes().getValue(attributeName);
-				if (attributeValue!=null) {
-					if (value.equals(attributeValue)) return startTag;
-					if (value.equalsIgnoreCase(attributeValue)) {
-						if (!valueCaseSensitive) return startTag;
-						if (source.isLoggingEnabled()) source.log(source.getRowColumnVector(pos).appendTo(new StringBuffer(200)).append(": StartTag with attribute ").append(attributeName).append("=\"").append(attributeValue).append("\" ignored during search because its case does not match search value \"").append(value).append('"').toString());
+			if (tag.begin>=pos) {
+				final StartTag startTag=(StartTag)tag;
+				if (startTag.getAttributes()!=null) {
+					final String attributeValue=startTag.getAttributes().getValue(attributeName);
+					if (attributeValue!=null) {
+						if (value.equals(attributeValue)) return startTag;
+						if (value.equalsIgnoreCase(attributeValue)) {
+							if (!valueCaseSensitive) return startTag;
+							if (source.logger.isInfoEnabled()) source.logger.info(source.getRowColumnVector(searchPos).appendTo(new StringBuffer(200)).append(": StartTag with attribute ").append(attributeName).append("=\"").append(attributeValue).append("\" ignored during search because its case does not match search value \"").append(value).append('"').toString());
+						}
 					}
 				}
 			}
-			pos=startTag.end+5; // next attribute value can't be less than 5 chars after last start tag
+			searchPos=tag.end+5; // next attribute value can't be less than 5 chars after last start tag
 		}
 		return null;
 	}
@@ -830,3 +640,4 @@ public final class StartTag extends Tag {
 		return findEndTag(nextStartTagsEndTag.end,nextStartTag,nextEndTag,checkForEmptyElementTag);  // recurse to see if this is the matching end tag
 	}
 }
+
