@@ -1,22 +1,22 @@
 // Jericho HTML Parser - Java based library for analysing and manipulating HTML
-// Version 2.2
-// Copyright (C) 2006 Martin Jericho
-// http://sourceforge.net/projects/jerichohtml/
+// Version 2.5
+// Copyright (C) 2007 Martin Jericho
+// http://jerichohtml.sourceforge.net/
 //
 // This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Lesser General Public
-// License as published by the Free Software Foundation; either
-// version 2.1 of the License, or (at your option) any later version.
-// http://www.gnu.org/copyleft/lesser.html
+// modify it under the terms of either one of the following licences:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
+// 1. The Eclipse Public License (EPL) version 1.0,
+// included in this distribution in the file licence-epl-1.0.html
+// or available at http://www.eclipse.org/legal/epl-v10.html
 //
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// 2. The GNU Lesser General Public License (LGPL) version 2.1 or later,
+// included in this distribution in the file licence-lgpl-2.1.txt
+// or available at http://www.gnu.org/licenses/lgpl.txt
+//
+// This library is distributed on an "AS IS" basis,
+// WITHOUT WARRANTY OF ANY KIND, either express or implied.
+// See the individual licence texts for more details.
 
 package au.id.jericho.lib.html;
 
@@ -129,8 +129,8 @@ final class SubCache {
 	}
 
 	protected StringBuffer appendTo(final StringBuffer sb) {
-		sb.append("Cache for TagType : ").append(tagType).append('\n');
-		for (int i=0; i<=lastIndex(); i++) sb.append(array[i]).append('\n');
+		sb.append("Cache for TagType : ").append(tagType).append(Config.NewLine);
+		for (int i=0; i<=lastIndex(); i++) sb.append(array[i]).append(Config.NewLine);
 		return sb;
 	}
 
@@ -201,14 +201,14 @@ final class SubCache {
 			stepCacheEntry=array[--index];
 			if (stepCacheEntry.pos<=tagPos) break;
 			if (stepCacheEntry.tag!=null) {
-				if (stepCacheEntry.tag.includeInSearch()) throw new SourceCacheEntryMissingInternalException(tagType,tag,this);
+				if (stepCacheEntry.tag.includeInSearch()) throw new SourceCacheEntryMissingInternalError(tagType,tag,this);
 				stepCacheEntry.previousCached=true;
 				stepCacheEntry.nextCached=true;
 			} else {
 				stepCacheEntry.removed=true; compactStartIndex=Math.min(compactStartIndex,stepCacheEntry.index);
 			}
 		}	
-		if (stepCacheEntry.pos!=tagPos) throw new FoundCacheEntryMissingInternalException(tagType,tag,this);
+		if (stepCacheEntry.pos!=tagPos) throw new FoundCacheEntryMissingInternalError(tagType,tag,this);
 		stepCacheEntry.nextCached=true;
 		compact(compactStartIndex);
 	}
@@ -247,14 +247,14 @@ final class SubCache {
 				stepCacheEntry=array[++index];
 				if (stepCacheEntry.pos>=tagPos) break;
 				if (stepCacheEntry.tag!=null) {
-					if (stepCacheEntry.tag.includeInSearch()) throw new SourceCacheEntryMissingInternalException(tagType,tag,this);
+					if (stepCacheEntry.tag.includeInSearch()) throw new SourceCacheEntryMissingInternalError(tagType,tag,this);
 					stepCacheEntry.previousCached=true;
 					stepCacheEntry.nextCached=true;
 				} else {
 					stepCacheEntry.removed=true; compactStartIndex=Math.min(compactStartIndex,stepCacheEntry.index);
 				}
 			}	
-			if (stepCacheEntry.pos!=tagPos) throw new FoundCacheEntryMissingInternalException(tagType,tag,this);
+			if (stepCacheEntry.pos!=tagPos) throw new FoundCacheEntryMissingInternalError(tagType,tag,this);
 		}
 		stepCacheEntry.previousCached=true;
 		compact(compactStartIndex);
@@ -349,20 +349,20 @@ final class SubCache {
 		array=newArray;
 	}
 
-	private static class CacheEntryMissingInternalException extends RuntimeException {
-		public CacheEntryMissingInternalException(final TagType tagType, final Tag tag, final SubCache subCache, final String message) {
+	private static class CacheEntryMissingInternalError extends AssertionError {
+		public CacheEntryMissingInternalError(final TagType tagType, final Tag tag, final SubCache subCache, final String message) {
 			super("INTERNAL ERROR: Inconsistent Cache State for TagType \""+tagType+"\" - "+message+' '+tag.getDebugInfo()+'\n'+subCache);
 		}
 	}
 
-	private static class SourceCacheEntryMissingInternalException extends CacheEntryMissingInternalException {
-		public SourceCacheEntryMissingInternalException(final TagType tagType, final Tag tag, final SubCache subCache) {
+	private static class SourceCacheEntryMissingInternalError extends CacheEntryMissingInternalError {
+		public SourceCacheEntryMissingInternalError(final TagType tagType, final Tag tag, final SubCache subCache) {
 			super(tagType,tag,subCache,"cache entry no longer found in source:");
 		}
 	}
 
-	private static class FoundCacheEntryMissingInternalException extends CacheEntryMissingInternalException {
-		public FoundCacheEntryMissingInternalException(final TagType tagType, final Tag tag, final SubCache subCache) {
+	private static class FoundCacheEntryMissingInternalError extends CacheEntryMissingInternalError {
+		public FoundCacheEntryMissingInternalError(final TagType tagType, final Tag tag, final SubCache subCache) {
 			super(tagType,tag,subCache,"missing cache entry for found tag");
 		}
 	}
